#include <stdio.h>
#include <sys/types.h>
#include <stdlib.h>

#include "common.h"
#include "matrix.h"
#include "proto.h"
#include "mesh.h"

/*
 * p->p_nodsno means the number of the basis functions
 */

void
integral_init(struct proto_t *p)
{
	struct integral_t *intg;
	int	i;

	if (p->p_intg)
		fatal("%s", __F);

	intg = (struct integral_t *) malloc(sizeof(struct integral_t));
	if (intg == NULL)
		memerr("%s", __F);

	intg->intg_jacob = (double *) malloc(sizeof(double) * p->p_intgno);
	if (intg->intg_jacob == NULL)
		memerr("%s", __F);
	intg->intg_weights = (double *) malloc(sizeof(double) * p->p_intgno);
	if (intg->intg_weights == NULL)
		memerr("%s", __F);

	intg->intg_coors = matrix(p->p_nodsno, p->p_dimen,"coors", SP_DOUBLE);
	intg->intg_intgcoors=matrix(p->p_intgno,p->p_dimen,"intgxy",SP_DOUBLE);
	intg->intg_trans = matrix(p->p_intgno, p->p_nodsno, "trans",SP_DOUBLE);
	intg->intg_unit  = matrix(p->p_nodsno, p->p_dimen, "unit", SP_DOUBLE);
	intg->intg_unitflag = (int *) malloc(sizeof(int) * p->p_nodsno);
	intg->intg_realintgcoors=
		matrix(p->p_intgno, p->p_dimen,  "real intg", SP_DOUBLE);

	if (intg->intg_unitflag == NULL)
		memerr("%s", __F);

	intg->intg_nodes = (nodno_t *) malloc(sizeof(nodno_t) * p->p_nodsno);
	if (intg->intg_nodes == NULL)
		memerr("%s", __F);

	intg->intg_deriv = (struct matrix_t **)
		malloc(sizeof(struct matrix_t *) * p->p_intgno);
	intg->intg_jacobm = (struct matrix_t **)
		malloc(sizeof(struct matrix_t *) * p->p_intgno);
	intg->intg_halfjacobm = (struct matrix_t **)
		malloc(sizeof(struct matrix_t *) * p->p_intgno);

	intg->intg_tmp = matrix(p->p_dimen, p->p_dimen, "tmp", SP_DOUBLE);

	for (i=0; i < p->p_intgno; i++) { /* for integration points */
		intg->intg_deriv[i]
			= matrix(p->p_dimen, p->p_nodsno,"deriv", SP_DOUBLE);
		intg->intg_jacobm[i]
			= matrix(p->p_dimen, p->p_dimen, "jacobm", SP_DOUBLE);
		intg->intg_halfjacobm[i]
			= matrix(p->p_dimen, p->p_nodsno,"halfj", SP_DOUBLE);
	}
	intg->intg_elemno = -1;	/* XXX MOST OMMISIBLE line */

	/*
	 * for the corner nodes
	 */
#if 0
	intg_hjatcorners = (struct matrix_t **)
		malloc(sizeof(struct matrix_t *) * p->p_nodsno);
	intg_derivatcorners = (struct matrix_t **)
		malloc(sizeof(struct matrix_t *) * p->p_nodsno);
	intg_jacobmatcorners = (struct matrix_t **)
		malloc(sizeof(struct matrix_t *) * p->p_nodsno);

	for (i=0; i < p->p_intgno; i++) { /* for integration points */
		intg->intg_derivatcorners[i]
			= matrix(p->p_dimen, p->p_nodsno,"deriv", SP_DOUBLE);
		intg->intg_jacobmatcorners[i]
			= matrix(p->p_dimen, p->p_dimen, "jacobm", SP_DOUBLE);
		intg->intg_hjatcorners[i]
			= matrix(p->p_dimen, p->p_nodsno,"halfj", SP_DOUBLE);
	}
#endif

	/*
	 * All malloc is over
	 */
	p->p_intg = intg;
	intg->intg_proto = p;

	return;
}
